# Copyright (C) 2004 Laurent Sansonetti
#
# Tengen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Tengen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Tengen; see the file COPYING.  If not,
# write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

require 'thread'

module Tengen
    class Player
        attr_reader :mode, :color, :actions, :stones, :captures
        
        class Color < String; end
        BLACK = Color.new("Black")
        WHITE = Color.new("White")
    
        class Human < self
            def play(move)
                Game::Engine.instance.play(color, move)
                @game.played(self, Action.new_move(move))
            end
    
            def pass
                Game::Engine.instance.pass(color)
                @game.played(self, Action.new_pass)
            end
    
            def resign
                @game.resigned(self)
            end

            def kind
                'Human'
            end
        end

        class Computer < self
            def genmove
                Game::Engine.instance.request_genmove(self)
            end
        
            def do_genmove
                case move = Game::Engine.instance.genmove(color)
                    when 'RESIGNED'
                        @game.resigned(self)
                    when 'PASS'
                        @game.played(self, Action.new_pass)
                    else
                        @game.played(self, Action.new_move(move))
                end
            end

            def kind
                'Computer'
            end
        end

        def initialize
            @captures = 0
            @actions = []
            @stones = []
        end

        def human?
            self.is_a?(Human)
        end
    
        def computer?
            self.is_a?(Computer) 
        end
    
        def enter(game, color)
            raise "Already playing a game" unless @game.nil?
            @game = game
            @color = color
            game.add_observer(self)
        end
    
        def update(game, player, action)
            engine = Tengen::Game::Engine.instance
            @stones = [engine.stones(@color)].flatten
            if player == self
                @actions << action
                @captures = engine.captures(@color)
            elsif not action.undo? and computer? 
                genmove
            end
        end
       
        def to_s
            "#{self.kind} (#{@color})"
        end
    end
end
